/** @file
 Entry and initial functions for Microwindows driver

;******************************************************************************
;* Copyright (c) 2012 - 2013, Insyde Software Corp. All Rights Reserved.
;*
;* You may not reproduce, distribute, publish, display, perform, modify, adapt,
;* transmit, broadcast, present, recite, release, license or otherwise exploit
;* any part of this publication in any form, by any means, without the prior
;* written permission of Insyde Software Corporation.
;*
;******************************************************************************

*/
#include <Uefi.h>
#include <Protocol/H2OWindow.h>

EFI_SIMPLE_TEXT_INPUT_EX_PROTOCOL  *mSimpleTextInEx = NULL;

H2O_WINDOW_PROTOCOL             mH2OWMProtocol = {
  sizeof (H2O_WINDOW_PROTOCOL),
  SendMessage,
  PostMessage,
  PostQuitMessage,
  GetMessage,
  PeekMessage,
  TranslateMessage,
  DispatchMessage,

  RegisterClass,
//UnRegisterClass,
  CreateWindowEx,
  DestroyWindow,
//GetWindowLong,
//SetWindowLong,
  GetWindowWord,
  SetWindowWord,
//GetClassLong,
  GetWindowText,
  SetWindowText,

  SetTextColor,
  SetBkColor,
  GetSysColor,
  SetSysColor,
  SetBkMode,
  SetROP2,
  SetPixel,
  MoveToEx,
  LineTo,
  Rectangle,
  FillRect,
  TextOut,
//ExtTextOut,
  DrawText,
//DrawDIB,
  SelectObject,
  GetStockObject,
  CreatePen,
  CreateSolidBrush,
  CreateCompatibleBitmap,
  DeleteObject,
  CreateCompatibleDC,
  DeleteDC,
  BitBlt,
//GetSystemPaletteEntries,

//WndSetSetDesktopWallpaper,
//WndSetCursor,
//WndRaiseWindow,
//WndLowerWindow,
//WndGetTopWindow,

  GetTickCount,
//Sleep,
  SetTimer,
  KillTimer,
  GetCursorPos,
#if 0
  SetRect,
  SetRectEmpty,
  CopyRect,
  IsRectEmpty,
  InflateRect,
  OffsetRect,
  PtInRect,
  IntersectRect,
  UnionRect,
  SubtractRect,
  EqualRect,
#endif
  CreateRectRgn,
  CreateRectRgnIndirect,
  SetRectRgn,
  CreateRoundRectRgn,
  CreateEllipticRgn,
  CreateEllipticRgnIndirect,
  OffsetRgn,
  GetRgnBox,
//GetRgnData,
//PtInRgn,
//RectInRgn,
  EqualRgn,
  CombineRgn,

//SelectClipRegion,
//ExtSelectClipRegion,

  DefWindowProc,
  CallWindowProc,

  MwSetKeyboardTranslator,
  GetDCEx,
  GetDC,
  GetWindowDC,
  ReleaseDC,
  BeginPaint,
  EndPaint,
  GetTextColor,
  SetTextAlign,
  GetTextAlign,
  GetPixel,
  Polyline,
//Ellipse,
//Pie,
  Polygon,
  PolyPolygon,
  MwSetTextCoding,
//TextOutW,
//ExtTextOutW,
//DrawTextA,
  GetSysColorBrush,
  SelectClipRgn,
  ExtSelectClipRgn,
  GetUpdateRgn,
  GetUpdateRect,
  StretchBlt,
  GetDeviceCaps,
//DrawFocusRect,
//TabbedTextOut,
//GetTabbedTextExtent,
//MulDiv,
  GetTextExtentPoint32,
//Sleep,
  CreatePolygonRgn,
  GetRegionData,
  PtInRegion,
  RectInRegion,
  PostThreadMessage,
  UnregisterClass,
  IsWindow,
  ShowWindow,
  InvalidateRect,
  InvalidateRgn,
  ValidateRect,
  ValidateRgn,
  UpdateWindow,
  GetFocus,
  SetFocus,
  SetForegroundWindow,
  SetActiveWindow,
  GetActiveWindow,
  BringWindowToTop,
  GetDesktopWindow,
  GetParent,
  EnableWindow,
  AdjustWindowRectEx,
  GetClientRect,
  GetWindowRect,
  ClientToScreen,
  ScreenToClient,
  MapWindowPoints,
//GetWindowLong,
  GetWindowLongPtr,
//SetWindowLong,
  SetWindowLongPtr,
//GlobalFindAtom,
//GlobalAddAtom,
  SetProp,
  GetProp,
  RemoveProp,
//GetClassLong,
  GetClassLongPtr,
  GetWindowTextLength,
  MoveWindow,
  GetCapture,
  SetCapture,
  ReleaseCapture,
  GetSystemMetrics,
  SystemParametersInfo,
  GetDlgItem,
  EnumChildWindows,
  SetWindowPos,
  GetClipBox,
  SetDCPenColor,
  SetDCBrushColor,
  MwRegisterHotKey,
  UnregisterHotKey,

  //
  // microwindows functions
  //
  MwOpen,
  GdCloseMouse,
  GdOpenMouse,
  MwFastFillRect,
  GdDrawImagePartToFit,
  GdLoadImageFromBuffer,
  GdLoadImageFromBuffer2,

  GdAddGopDevice,
  GdAddEfiKeyData,
  GdAddPointerData,
};




/**
 Initialize Microwindows driver.

 @param[in] ImageHandle         The image handle.
 @param[in] SystemTable         The system table.

 @retval EFI_SUCCESS            The Microwindows module is initialized correctly.

 @return Other
**/
EFI_STATUS
EFIAPI
InitializeMicrowindows (
  IN EFI_HANDLE                 ImageHandle,
  IN EFI_SYSTEM_TABLE           *SystemTable
  )
{
  EFI_STATUS                    Status;


  Status = gBS->InstallProtocolInterface (
                  &ImageHandle,
                  &gH2OWindowProtocolGuid,
                  EFI_NATIVE_INTERFACE,
                  &mH2OWMProtocol
                  );

  return Status;
}

